@@ -15,7 +15,7 @@ def _run(ctx, payload: dict, box: str):
     run_python_internal(
         ctx=ctx,
         runnable=_impl_arm_path(),
-        box=gateway,
+        box=box,
         image=_gateway_image(),
         env=(),
         passenv=(),
@@ -39,9 +39,8 @@ def _require_netname(ctx) -> str:
 def _resolve_gateway_for_command(ctx, dut):
     """Resolve gateway from command-level --dut option or group-level stored gateway"""
     if dut:
-        from ..dut_storage import get_dut_ip
-        local_ip = get_dut_ip(dut)
-        return local_ip if local_ip else dut
+        from ..dut_storage import resolve_and_validate_dut
+        return resolve_and_validate_dut(ctx, dut)
     # Fall back to gateway stored by the group command
     return getattr(ctx.obj, "gateway", None) or get_default_gateway(ctx)
 
@@ -75,9 +74,8 @@ def arm(ctx, box,
     # Only resolve gateway if dut is provided at group level
     # Otherwise, let subcommands resolve it
     if dut:
-        from ..dut_storage import get_dut_ip
-        local_ip = get_dut_ip(dut)
-        gw = local_ip if local_ip else dut
+        from ..dut_storage import resolve_and_validate_dut
+        gw = resolve_and_validate_dut(ctx, dut)
         setattr(ctx.obj, "gateway", gw)
     else:
         # Don't set gateway - let subcommands handle it
