@@ -29,21 +29,11 @@ def _require_netname(ctx) -> str:
 
 
 def _resolve_gateway(ctx, box, dut):
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Check if box is a local DUT name first
-    if box:
-        local_ip = get_dut_ip(box)
-        if local_ip:
-            return local_ip
-
-    # Check if dut is a local DUT name
-    if dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            return local_ip
-
-    return box or dut or get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+    return resolve_and_validate_dut(ctx, box_name)
 
 
 def _run_net_py(ctx: click.Context, gateway: str, *args: str) -> list[dict]:
@@ -137,7 +127,7 @@ def battery(ctx, box, dut, netname):
     # When there's a subcommand, let it handle gateway resolution with its own --dut option
     if ctx.invoked_subcommand is None:
         gateway = _resolve_gateway(ctx, box, dut)
-        _list_battery_nets(ctx, box)   
+        _list_battery_nets(ctx, gateway)   
 
 @battery.command()
 @click.argument('MODE_TYPE', required=False, type=click.Choice(('static', 'dynamic')))
