@@ -9,7 +9,7 @@ import json
 from texttable import Texttable
 from ..context import get_default_gateway, get_impl_path
 from ..python.commands import run_python_internal
-from ..dut_storage import get_dut_ip
+from ..dut_storage import resolve_and_validate_dut
 
 @click.group(name='ble')
 def ble():
@@ -45,13 +45,11 @@ def scan(ctx, box,
     """
         Scan for BLE devices
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     scan_args = {
         'action': 'scan',
@@ -79,13 +77,11 @@ def info(ctx, box,
     """
         Get BLE device information
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     run_python_internal(
         ctx, get_impl_path('ble_info.py'), dut,
@@ -105,13 +101,11 @@ def connect(ctx, box,
     """
         Connect to a BLE device
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     connect_args = {
         'action': 'connect',
@@ -136,13 +130,11 @@ def disconnect(ctx, box,
     """
         Disconnect from a BLE device
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     run_python_internal(
         ctx, get_impl_path('ble_disconnect.py'), dut,
