@@ -20,20 +20,21 @@ def boxes(ctx):
         # Default behavior: list DUTs
         table = Texttable()
         table.set_deco(Texttable.HEADER)
-        table.set_cols_dtype(['t', 't', 't'])
-        table.set_cols_align(["l", "l", "l"])
-        table.add_row(['name', 'ip', 'user'])
+        table.set_cols_dtype(['t', 't', 't', 't'])
+        table.set_cols_align(["l", "l", "l", "l"])
+        table.add_row(['name', 'ip', 'version', 'user'])
 
         # Show only saved local DUTs
         saved_duts = list_duts()
         for name, dut_info in sorted(saved_duts.items()):
             if isinstance(dut_info, dict):
                 ip = dut_info.get('ip', 'unknown')
+                version = dut_info.get('version', '-')
                 user = dut_info.get('user', '-')
-                table.add_row([name, ip, user])
+                table.add_row([name, ip, version, user])
             else:
                 # Handle simple string format (IP only)
-                table.add_row([name, dut_info, '-'])
+                table.add_row([name, dut_info, '-', '-'])
 
         if not saved_duts:
             click.echo("No local boxes found. Add one with: lager boxes add --name <NAME> --ip <IP>")
@@ -45,8 +46,9 @@ def boxes(ctx):
 @click.option('--name', required=True, help='Name to assign to the box')
 @click.option('--ip', required=True, help='IP address of the box')
 @click.option('--user', required=False, help='Username for SSH connection (defaults to lagerdata)')
+@click.option('--version', required=False, help='Gateway version/branch (e.g., staging, main)')
 @click.option('--yes', is_flag=True, help='Confirm the action without prompting.')
-def add(name, ip, user, yes):
+def add(name, ip, user, version, yes):
     """
         Add a box configuration
     """
@@ -150,10 +152,12 @@ def add(name, ip, user, yes):
                 click.echo("Cancelled. DUT not added.")
                 return
 
-    add_dut(name, ip, user)
+    add_dut(name, ip, user, version)
     success_msg = f"Added DUT '{name}' with IP '{ip}'"
     if user:
         success_msg += f" (user: {user})"
+    if version:
+        success_msg += f" (version: {version})"
     click.echo(click.style(success_msg, fg='green'))
 
 
@@ -198,15 +202,16 @@ def delete(name, yes):
 @click.option('--name', required=True, help='Name of the box to edit')
 @click.option('--ip', required=False, help='New IP address for the box')
 @click.option('--user', required=False, help='New username for SSH connection')
+@click.option('--version', required=False, help='New gateway version/branch')
 @click.option('--new-name', required=False, help='New name for the box')
 @click.option('--yes', is_flag=True, help='Confirm the action without prompting.')
-def edit(name, ip, user, new_name, yes):
+def edit(name, ip, user, version, new_name, yes):
     """
         Edit a box configuration
     """
     # Check if at least one change is specified
-    if ip is None and new_name is None and user is None:
-        click.echo(click.style("Error: You must specify at least one change (--ip, --user, or --new-name)", fg='red'), err=True)
+    if ip is None and new_name is None and user is None and version is None:
+        click.echo(click.style("Error: You must specify at least one change (--ip, --user, --version, or --new-name)", fg='red'), err=True)
         raise click.Abort()
 
     # Check if DUT exists
@@ -220,13 +225,16 @@ def edit(name, ip, user, new_name, yes):
     if isinstance(dut_info, dict):
         current_ip = dut_info.get('ip')
         current_user = dut_info.get('user')
+        current_version = dut_info.get('version')
     else:
         current_ip = dut_info
         current_user = None
+        current_version = None
 
     # Determine new values (keep old if not specified)
     updated_ip = ip if ip else current_ip
     updated_user = user if user is not None else current_user
+    updated_version = version if version is not None else current_version
     updated_name = new_name if new_name else name
 
     # Validate new IP if specified
@@ -275,6 +283,11 @@ def edit(name, ip, user, new_name, yes):
                 changes.append(f"user: {current_user} → {updated_user}")
             else:
                 changes.append(f"user: (none) → {updated_user}")
+        if version is not None:
+            if current_version:
+                changes.append(f"version: {current_version} → {updated_version}")
+            else:
+                changes.append(f"version: (none) → {updated_version}")
 
         for change in changes:
             click.echo(f"  Change:  {change}")
@@ -290,7 +303,7 @@ def edit(name, ip, user, new_name, yes):
         delete_dut(name)
 
     # Add/update with new values
-    add_dut(updated_name, updated_ip, updated_user)
+    add_dut(updated_name, updated_ip, updated_user, updated_version)
 
     # Build success message
     changes_made = []
@@ -300,6 +313,8 @@ def edit(name, ip, user, new_name, yes):
         changes_made.append(f"changed IP to '{updated_ip}'")
     if user is not None:
         changes_made.append(f"changed user to '{updated_user}'")
+    if version is not None:
+        changes_made.append(f"changed version to '{updated_version}'")
 
     success_msg = f"Updated DUT"
     if changes_made:
@@ -491,4 +506,112 @@ def import_duts(file, merge, yes):
     else:
         # Replace: use only import DUTs
         save_duts(import_duts)
-        click.echo(click.style(f"✓ Successfully imported {len(import_duts)} DUT(s) from {file}", fg='green'))
\ No newline at end of file
+        click.echo(click.style(f"✓ Successfully imported {len(import_duts)} DUT(s) from {file}", fg='green'))
+
+
+@boxes.command('sync')
+@click.option('--port', default=5000, help='Gateway HTTP port (default: 5000)')
+@click.option('--timeout', default=5, type=float, help='HTTP request timeout in seconds (default: 5)')
+def sync(port, timeout):
+    """
+        Sync gateway versions from all boxes
+
+        Queries each gateway via HTTP to read its current CLI version
+        and updates the local configuration with the latest versions.
+    """
+    import requests
+    from ..dut_storage import update_dut_version
+
+    saved_duts = list_duts()
+
+    if not saved_duts:
+        click.echo("No boxes found. Add boxes with: lager boxes add --name <NAME> --ip <IP>")
+        return
+
+    click.echo(click.style(f'\nSyncing versions from {len(saved_duts)} gateway(s)...', fg='blue', bold=True))
+    click.echo()
+
+    updated_count = 0
+    failed_count = 0
+    unchanged_count = 0
+
+    for name, dut_info in sorted(saved_duts.items()):
+        # Extract IP address
+        if isinstance(dut_info, dict):
+            ip = dut_info.get('ip', 'unknown')
+            current_version = dut_info.get('version')
+        else:
+            ip = dut_info
+            current_version = None
+
+        # Skip if IP is invalid
+        if ip == 'unknown':
+            click.echo(f"  {name}: ", nl=False)
+            click.secho('SKIPPED (no IP)', fg='yellow')
+            failed_count += 1
+            continue
+
+        # Query gateway for version
+        click.echo(f"  {name} ({ip}): ", nl=False)
+
+        try:
+            # Make HTTP request to gateway's CLI version endpoint
+            url = f'http://{ip}:{port}/cli-version'
+            response = requests.get(url, timeout=timeout)
+
+            if response.status_code == 200:
+                data = response.json()
+                gateway_version = data.get('gateway_version')
+
+                if gateway_version:
+                    # Check if version changed
+                    if current_version == gateway_version:
+                        click.secho(f'{gateway_version} (unchanged)', fg='white', dim=True)
+                        unchanged_count += 1
+                    else:
+                        # Update local storage
+                        if update_dut_version(name, gateway_version):
+                            if current_version:
+                                click.secho(f'{current_version} → {gateway_version}', fg='green')
+                            else:
+                                click.secho(f'{gateway_version} (new)', fg='green')
+                            updated_count += 1
+                        else:
+                            click.secho('FAILED (update error)', fg='red')
+                            failed_count += 1
+                else:
+                    click.secho('FAILED (no version in response)', fg='red')
+                    failed_count += 1
+
+            elif response.status_code == 404:
+                click.secho('NOT AVAILABLE (endpoint not found - update gateway)', fg='yellow')
+                failed_count += 1
+            else:
+                click.secho(f'FAILED (HTTP {response.status_code})', fg='red')
+                failed_count += 1
+
+        except requests.exceptions.Timeout:
+            click.secho(f'TIMEOUT (after {timeout}s)', fg='red')
+            failed_count += 1
+        except requests.exceptions.ConnectionError:
+            click.secho('CONNECTION FAILED', fg='red')
+            failed_count += 1
+        except Exception as e:
+            click.secho(f'ERROR ({type(e).__name__})', fg='red')
+            failed_count += 1
+
+    # Summary
+    click.echo()
+    click.echo(click.style('Summary:', fg='blue', bold=True))
+    click.echo(f"  Updated:   {updated_count}")
+    click.echo(f"  Unchanged: {unchanged_count}")
+    click.echo(f"  Failed:    {failed_count}")
+
+    if updated_count > 0:
+        click.echo()
+        click.secho(f'✓ Successfully synced {updated_count} gateway version(s)', fg='green')
+
+    if failed_count > 0:
+        click.echo()
+        click.echo(click.style('Note:', fg='yellow') + ' Failed gateways may need to be updated first:')
+        click.echo('  lager update --box <name> --version staging')
\ No newline at end of file
