@@ -1181,22 +1181,56 @@ class DirectHTTPSession:
         """
         url = f'{self.base_url}/python'
 
-        try:
-            response = self.session.post(
-                url,
-                files=files,
-                stream=True,
-                timeout=(7, 320)  # Connect timeout: 7s, Read timeout: 320s
-            )
-            return response
-        except requests.exceptions.ConnectionError as e:
+        # Retry logic for multipart upload connection issues
+        max_retries = 3
+        last_error = None
+
+        for attempt in range(max_retries):
+            try:
+                # Disable keep-alive to prevent connection reuse issues with multipart uploads
+                headers = {'Connection': 'close'}
+
+                # Reset file positions if they are BytesIO objects
+                for name, value in files:
+                    if hasattr(value, 'seek') and hasattr(value, 'read'):
+                        # It's a file-like object, reset position
+                        value.seek(0)
+                    elif isinstance(value, tuple) and len(value) >= 2:
+                        # It's a tuple (filename, file_obj, ...), check the file object
+                        if hasattr(value[1], 'seek'):
+                            value[1].seek(0)
+
+                response = self.session.post(
+                    url,
+                    files=files,
+                    headers=headers,
+                    stream=True,
+                    timeout=(7, 320)  # Connect timeout: 7s, Read timeout: 320s
+                )
+                return response
+            except requests.exceptions.ConnectionError as e:
+                last_error = e
+                if attempt < max_retries - 1:
+                    # Wait a bit before retrying
+                    import time
+                    time.sleep(0.5)
+                    continue
+                # Last attempt failed - exit cleanly without traceback
+                click.secho(f'Could not connect to gateway at {self.gateway_ip}', fg='red', err=True)
+                click.secho('Ensure the gateway is online and reachable via your network', fg='yellow', err=True)
+                click.secho('(Tailscale, VPN, or local network connection required)', fg='yellow', err=True)
+                import sys
+                sys.exit(1)
+            except requests.exceptions.Timeout:
+                click.secho(f'Connection to gateway at {self.gateway_ip} timed out', fg='red', err=True)
+                import sys
+                sys.exit(1)
+
+        # Should not reach here, but just in case
+        if last_error:
             click.secho(f'Could not connect to gateway at {self.gateway_ip}', fg='red', err=True)
-            click.secho('Ensure the gateway is online and reachable via your network', fg='yellow', err=True)
-            click.secho('(Tailscale, VPN, or local network connection required)', fg='yellow', err=True)
-            raise
-        except requests.exceptions.Timeout:
-            click.secho(f'Connection to gateway at {self.gateway_ip} timed out', fg='red', err=True)
-            raise
+            import sys
+            sys.exit(1)
 
     def kill_python(self, gateway, lager_process_id, sig=signal.SIGTERM):
         """
