@@ -8,6 +8,8 @@ import click
 import json
 import io
 import requests
+import signal
+import sys
 from contextlib import redirect_stdout
 from ..context import get_default_gateway, get_impl_path, get_default_net
 from ..python.commands import run_python_internal
@@ -29,23 +31,18 @@ def _resolve_dut_with_username(ctx, dut):
     Returns:
         Tuple of (ip_address, username)
     """
-    dut_name = None
-    dut_ip = dut
+    from ..dut_storage import resolve_and_validate_dut
 
-    if dut:
-        # Try to resolve as DUT name first
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut_ip = saved_ip
-            dut_name = dut  # Original parameter was a DUT name
-        else:
-            # Parameter was an IP, try reverse lookup
-            dut_name = get_dut_name_by_ip(dut)
+    # Resolve and validate the DUT name/IP
+    dut_ip = resolve_and_validate_dut(ctx, dut)
+
+    # Determine DUT name for username lookup
+    # If dut was provided and is not an IP, it's the DUT name
+    if dut and not dut.replace('.', '').isdigit():
+        dut_name = dut
     else:
-        # Use default gateway
-        dut_ip = get_default_gateway(ctx)
-        if dut_ip:
-            dut_name = get_dut_name_by_ip(dut_ip)
+        # It was an IP or None, try reverse lookup
+        dut_name = get_dut_name_by_ip(dut_ip)
 
     # Get username (defaults to 'lagerdata' if not found)
     username = get_dut_user(dut_name) if dut_name else 'lagerdata'
@@ -360,7 +357,7 @@ def gdbserver(ctx, box,
 
     if already_connected:
         if not quiet and not json_output:
-            click.echo("Already connected. Disconnecting before reconnecting...")
+            click.echo("Already connected. Disconnecting before reconnecting...", err=True)
         try:
             client.disconnect(debug_net, keep_jlink_running=False)
         except Exception as e:
@@ -368,6 +365,11 @@ def gdbserver(ctx, box,
             # This is expected and can be safely ignored
             pass
 
+        # Wait for GDB server to fully shut down before reconnecting
+        # This prevents "No debugger connection found" errors on reconnect
+        import time
+        time.sleep(1.0)
+
     # Connect to debugger and start GDB server
     try:
         result = client.connect(debug_net, speed=speed, force=force, halt=halt, gdb=True, gdb_port=gdb_port)
@@ -427,11 +429,59 @@ def gdbserver(ctx, box,
 
     # Handle post-connect actions
     if rtt or rtt_reset:
+        # Wait for GDB server to fully initialize before attempting reset/RTT
+        # This prevents "No debugger connection found" errors
+        # The server takes ~2-3s to fully initialize:
+        # 1. Start the JLinkGDBServer process (~500ms)
+        # 2. Write PID file and detect running status (~500ms)
+        # 3. Initialize GDB server connection (~500ms)
+        # 4. Establish target connection and detect device (~1s)
+        # Note: Time varies based on probe type, target, and system load
+        import time
+        time.sleep(3.0)
+        # Ignore SIGPIPE to prevent "Exception ignored in: <_io.TextIOWrapper>" messages
+        # when the pipe is broken (e.g., defmt-print exits before we finish)
+        # On Windows, SIGPIPE doesn't exist, so we need to handle that
+        try:
+            signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+        except AttributeError:
+            # Windows doesn't have SIGPIPE
+            pass
+
         # If --rtt-reset, reset the device first to capture boot sequence
         if rtt_reset:
             if not quiet:
                 click.echo("Resetting device to capture boot sequence...", err=True)
-            client.reset(debug_net, halt=False)
+            try:
+                client.reset(debug_net, halt=False)
+            except requests.exceptions.HTTPError as e:
+                # Parse error response
+                error_msg = "Unknown error"
+                try:
+                    error_json = e.response.json()
+                    error_msg = error_json.get('error', str(e))
+                except:
+                    error_msg = str(e)
+
+                click.secho(f"Error: Failed to reset device: {error_msg}", fg='red', err=True)
+
+                # Provide troubleshooting steps
+                if "No debugger connection found" in error_msg or "400" in str(e):
+                    click.secho("\nThis can happen if:", fg='yellow', err=True)
+                    click.secho("  • GDB server didn't fully initialize (timing issue)", fg='yellow', err=True)
+                    click.secho("  • No physical target device connected", fg='yellow', err=True)
+                    click.secho("  • Target device is not powered", fg='yellow', err=True)
+                    click.secho("\nTry:", fg='cyan', err=True)
+                    click.secho("  • Running the command again (may be a timing issue)", fg='cyan', err=True)
+                    click.secho("  • Using --rtt instead of --rtt-reset if device is already running", fg='cyan', err=True)
+                    click.secho("  • Verifying target is connected and powered", fg='cyan', err=True)
+
+                client.close()
+                ctx.exit(1)
+            except Exception as e:
+                click.secho(f"Error: Failed to reset device: {e}", fg='red', err=True)
+                client.close()
+                ctx.exit(1)
 
             # Wait for device to reset and firmware to boot
             # This delay ensures:
@@ -453,12 +503,23 @@ def gdbserver(ctx, box,
             for chunk in client.rtt(net=debug_net, channel=0, timeout=None):
                 # Write directly to stdout in binary mode for maximum performance
                 import sys
-                sys.stdout.buffer.write(chunk)
-                sys.stdout.buffer.flush()
+                try:
+                    sys.stdout.buffer.write(chunk)
+                    sys.stdout.buffer.flush()
+                except BrokenPipeError:
+                    # Pipe closed (e.g., defmt-print exited) - exit gracefully
+                    if not quiet:
+                        click.echo("\nRTT stream stopped (pipe closed)", err=True)
+                    break
         except KeyboardInterrupt:
             # User pressed Ctrl+C - graceful exit
             if not quiet:
                 click.echo("\nRTT stream stopped", err=True)
+        except BrokenPipeError:
+            # Pipe closed - exit gracefully without error message
+            # This happens when piped command (e.g., defmt-print) exits first
+            if not quiet:
+                click.echo("\nRTT stream stopped (pipe closed)", err=True)
         except Exception as e:
             click.secho(f"\nRTT stream error: {e}", fg='red', err=True)
         finally:
