@@ -79,18 +79,24 @@ def save_duts(duts: Dict[str, str]) -> None:
         json.dump(data, f, indent=2)
 
 
-def add_dut(name: str, ip: str, user: Optional[str] = None) -> None:
+def add_dut(name: str, ip: str, user: Optional[str] = None, version: Optional[str] = None) -> None:
     """Add a DUT to the local storage.
 
     Args:
         name: DUT name
         ip: IP address
-        user: Optional username (if None, stores in legacy format)
+        user: Optional username (if None and version is None, stores in legacy format)
+        version: Optional version/branch name (e.g., "staging", "main")
     """
     duts = load_duts()
-    if user:
-        # New format with user
-        duts[name] = {"ip": ip, "user": user}
+    if user or version:
+        # New format with user and/or version
+        dut_dict = {"ip": ip}
+        if user:
+            dut_dict["user"] = user
+        if version:
+            dut_dict["version"] = version
+        duts[name] = dut_dict
     else:
         # Legacy format (just IP string)
         duts[name] = ip
@@ -127,6 +133,49 @@ def get_dut_user(name: str) -> Optional[str]:
     return None
 
 
+def get_dut_version(name: str) -> Optional[str]:
+    """Get the version for a named DUT.
+
+    Args:
+        name: DUT name
+
+    Returns:
+        Version if stored, None otherwise
+    """
+    duts = load_duts()
+    dut_info = duts.get(name)
+    if isinstance(dut_info, dict):
+        return dut_info.get("version")
+    # Legacy format (string IP) has no version
+    return None
+
+
+def update_dut_version(name: str, version: str) -> bool:
+    """Update the version for a named DUT.
+
+    Args:
+        name: DUT name
+        version: Version/branch name (e.g., "staging", "main")
+
+    Returns:
+        True if updated, False if DUT not found
+    """
+    duts = load_duts()
+    if name not in duts:
+        return False
+
+    dut_info = duts[name]
+    if isinstance(dut_info, dict):
+        # Update version in existing dict
+        dut_info["version"] = version
+    else:
+        # Upgrade from legacy format to dict format
+        duts[name] = {"ip": dut_info, "version": version}
+
+    save_duts(duts)
+    return True
+
+
 def get_dut_name_by_ip(ip: str) -> Optional[str]:
     """Reverse lookup: find DUT name by IP address.
 
