@@ -7,18 +7,13 @@ from ..python.commands import run_python_internal
 
 def _resolve_gateway(ctx, dut):
     """Resolve DUT name to IP address if it's a local DUT."""
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
     # Get the gateway/dut identifier (either from parameter or default)
-    gateway_id = dut or get_default_gateway(ctx)
+    gateway_id = dut or None
 
-    # Try to resolve it as a local DUT name
-    if gateway_id:
-        local_ip = get_dut_ip(gateway_id)
-        if local_ip:
-            return local_ip
-
-    return gateway_id
+    # Resolve and validate the DUT name (shows helpful error if invalid)
+    return resolve_and_validate_dut(ctx, gateway_id)
 
 
 def _run_eload_script(ctx, box: str, args: list):
@@ -26,7 +21,7 @@ def _run_eload_script(ctx, box: str, args: list):
     run_python_internal(
         ctx,
         get_impl_path('eload.py'),
-        gateway,
+        box,
         image='',
         env=(),
         passenv=(),
