@@ -37,9 +37,6 @@ YELLOW = '\033[93m'
 RED = '\033[91m'
 RESET = '\033[0m'
 
-# Add gateway python path
-sys.path.insert(0, '/opt/lager/gateway/python')
-
 from lager.debug import (
     connect_jlink,
     disconnect,
