@@ -19,10 +19,12 @@ SUPPORTED_USB: Dict[str, Dict[str, str | List[str] | None]] = {
     "Rigol_DP811":       {"vid": "1ab1", "pid": "????", "net_type": ["power-supply"]}, # PID was e011 but that was wrong
     "Rigol_DP821":       {"vid": "1ab1", "pid": "0e11", "net_type": ["power-supply"]},
     "Rigol_DP831":       {"vid": "1ab1", "pid": "0e31", "net_type": ["power-supply"]},
+    "Rigol_DP832":       {"vid": "1ab1", "pid": "0e11", "net_type": ["power-supply"]}, # Same VID:PID as DP821, differentiated by serial
     "EA_PSB_10080_60":   {"vid": "232e", "pid": "0053", "net_type": ["power-supply", "solar"]},
     "EA_PSB_10060_60":   {"vid": "232e", "pid": "0053", "net_type": ["power-supply", "solar"]},
     "KEYSIGHT_E36233A":  {"vid": "2a8d", "pid": "3302", "net_type": ["power-supply"]}, # Need to figure out actual PID/VID, 2a8d:3302, don't know which one this corresponds to
     "KEYSIGHT_E36313A":  {"vid": "2a8d", "pid": "3302", "net_type": ["power-supply"]}, # Need to figure out actual PID/VID
+    "KEYSIGHT_E36312A":  {"vid": "2a8d", "pid": "1102", "net_type": ["power-supply"]},
 
     # battery
     "Keithley_2281S":    {"vid": "05e6", "pid": "2281", "net_type": ["battery", "power-supply"]},
@@ -36,9 +38,10 @@ SUPPORTED_USB: Dict[str, Dict[str, str | List[str] | None]] = {
     # "LabJack_T7_Pro":    {"vid": "0cd5", "pid": "0000", "net_type": ["gpio", "adc", "dac"]},
 
     # debug
-    "J-Link":            {"vid": "1366", "pid": "1024", "net_type": ["debug"]},
-    "J-Link_Plus":       {"vid": "1366", "pid": "0101", "net_type": ["debug"]},
-    "Flasher_ARM":       {"vid": "1366", "pid": "0503", "net_type": ["debug"]},
+    "J-Link":                       {"vid": "1366", "pid": "1024", "net_type": ["debug"]},
+    "J-Link_Plus":                  {"vid": "1366", "pid": "0101", "net_type": ["debug"]},
+    "J-Link":                       {"vid": "1366", "pid": "1020", "net_type": ["debug"]},
+    "Flasher_ARM":                  {"vid": "1366", "pid": "0503", "net_type": ["debug"]},
 
     # usb
     "Acroname_8Port":     {"vid": "24ff", "pid": "0013", "net_type": ["usb"]},
@@ -64,6 +67,8 @@ SUPPORTED_USB: Dict[str, Dict[str, str | List[str] | None]] = {
     # uart
     "Prolific_USB_Serial": {"vid": "067b", "pid": "23a3", "net_type": ["uart"]},
     "SiLabs_CP210x": {"vid": "10c4", "pid": "ea60", "net_type": ["uart"]},
+    "FTDI_FT232R": {"vid": "0403", "pid": "6001", "net_type": ["uart"]},
+    "FTDI_FT4232H": {"vid": "0403", "pid": "6011", "net_type": ["uart"]},
 }
 
 # ── Channel maps ────────────────────────────────────────────
@@ -73,10 +78,12 @@ CHANNEL_MAPS = {
     "Rigol_DP811":            {"power-supply": ["1"]},
     "Rigol_DP821":            {"power-supply": ["1", "2"]},
     "Rigol_DP831":            {"power-supply": ["1", "2", "3"]},
+    "Rigol_DP832":            {"power-supply": ["1", "2", "3"]},
     "EA_PSB_10080_60":        {"power-supply": ["1"], "solar": ["1"]},
     "EA_PSB_10060_60":        {"power-supply": ["1"], "solar": ["1"]},
     "Keysight_E36233A":       {"power-supply": ["1", "2"]},
     "Keysight_E36313A":       {"power-supply": ["1", "2", "3"]},
+    "KEYSIGHT_E36312A":       {"power-supply": ["1", "2", "3"]},
 
     # battery
     "Keithley_2281S":         {"power-supply": ["1"], "battery": ["1"]},
@@ -103,6 +110,7 @@ CHANNEL_MAPS = {
     # debug
     "J-Link":                 {"debug": ["DEVICE_TYPE"]},
     "J-Link_Plus":            {"debug": ["DEVICE_TYPE"]},
+    "J-Link":                 {"debug": ["DEVICE_TYPE"]},
     "Flasher_ARM":            {"debug": ["DEVICE_TYPE"]},
 
     # usb
@@ -120,7 +128,7 @@ CHANNEL_MAPS = {
     # "Logitech_BRIO":          {"webcam": ["NA"]},
 
     # watt-meter
-    "Yocto_Watt":             {"watt-meter": ["1"]},
+    "Yocto_Watt":             {"watt-meter": ["0"]},
 
     # thermocouple
     "Phidget":                {"thermocouple": ["1"]},
@@ -337,10 +345,10 @@ def _get_tty_for_usb_serial(serial_number: Optional[str] = None) -> Optional[str
 
 _VIDPID_TO_NAME: Dict[tuple[str, str], str] = {}
 # Special handling for instruments with duplicate VID:PID
-# Skip Rigol_DL3021 as it shares PID with DP821 and will be handled by serial number
+# Skip instruments that share VID:PID 1ab1:0e11 - they need serial-based detection
 for _name, meta in SUPPORTED_USB.items():
-    if _name == "Rigol_DL3021":
-        continue  # Handle this specially by serial number
+    if _name in ("Rigol_DL3021", "Rigol_DP832"):
+        continue  # Handle these specially by serial number (share VID:PID with DP821)
     _VIDPID_TO_NAME[(meta["vid"].lower(), meta["pid"].lower())] = _name
 
 
@@ -387,10 +395,16 @@ def _scan_usb() -> List[dict]:
         # Special handling for Rigol instruments with same VID:PID (1ab1:0e11)
         # Differentiate by serial number prefix
         if vid == "1ab1" and pid == "0e11" and serial:
-            if serial.upper().startswith("DL3"):
+            serial_upper = serial.upper()
+            if serial_upper.startswith("DL3"):
                 meta_name = "Rigol_DL3021"  # DL3000 series electronic load
-            elif serial.upper().startswith("DP8"):
-                meta_name = "Rigol_DP821"  # DP800 series power supply
+            elif serial_upper.startswith("DP8B") or serial_upper.startswith("DP83"):
+                meta_name = "Rigol_DP832"  # DP832/DP832A - 3 channel power supply
+            elif serial_upper.startswith("DP82"):
+                meta_name = "Rigol_DP821"  # DP821 - 2 channel power supply
+            elif serial_upper.startswith("DP8"):
+                # Generic DP8xx - default to DP821
+                meta_name = "Rigol_DP821"
             else:
                 # Default to power supply if we can't determine from serial
                 meta_name = "Rigol_DP821"
@@ -442,6 +456,10 @@ def infer_instrument_from_address(address: str) -> str:
             serial = parts[3].upper()
             if serial.startswith("DL3"):
                 return "Rigol_DL3021"
+            elif serial.startswith("DP8B") or serial.startswith("DP83"):
+                return "Rigol_DP832"
+            elif serial.startswith("DP82"):
+                return "Rigol_DP821"
             elif serial.startswith("DP8"):
                 return "Rigol_DP821"
 
