@@ -9,7 +9,7 @@ from texttable import Texttable
 from ..context import get_impl_path
 from ..python.commands import run_python_internal
 from ..context import get_default_gateway
-from ..dut_storage import get_dut_ip
+from ..dut_storage import resolve_and_validate_dut
 from collections import defaultdict
 
 import io
@@ -23,18 +23,9 @@ _MULTI_HUBS = {"LabJack_T7", "Acroname_8Port", "Acroname_4Port"}
 @click.pass_context
 def instruments(ctx, box: str | None, dut: str | None = None) -> None:
     """List attached instruments"""
-    # Resolve box/dut (box takes precedence, dut is for backward compatibility)
-    resolved = box or dut
-
-    # Check if resolved is a local DUT name first, then gateway
-    if resolved:
-        local_ip = get_dut_ip(resolved)
-        if local_ip:
-            dut = local_ip
-        else:
-            dut = resolved
-    else:
-        dut = get_default_gateway(ctx)
+    # Resolve and validate the box/dut name
+    box_name = box or dut
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     buf = io.StringIO()
     try:
