@@ -20,7 +20,6 @@ from . import __version__
 from .config import read_config_file
 from .context import LagerContext
 
-from .gateway.commands import _gateway
 from .adc.commands import adc
 from .ble.commands import ble
 from .debug.commands import _debug
@@ -80,7 +79,6 @@ def cli(ctx=None, see_version=None, debug=False, colorize=False, interpreter=Non
     else:
         setup_context(ctx, debug, colorize, interpreter)
 
-cli.add_command(_gateway)
 cli.add_command(adc)
 cli.add_command(ble)
 cli.add_command(_debug)
