@@ -138,31 +138,30 @@ INSTRUMENT_NET_MAP: dict[str, list[str]] = {
 def _run_net_py(ctx: click.Context, dut: str, *net_args: str) -> str:
     """
     Run `net.py …` via run_python_internal and capture stdout.
-    Suppresses SystemExit raised by that script.
     """
-    buf = io.StringIO()
+    from ..python.commands import run_python_internal_get_output
+
     try:
-        with redirect_stdout(buf):
-            run_python_internal(
-                ctx,
-                get_impl_path("net.py"),
-                dut,
-                image="",
-                env={},
-                passenv=(),
-                kill=False,
-                download=(),
-                allow_overwrite=False,
-                signum="SIGTERM",
-                timeout=30,  # 30 second timeout to prevent hanging
-                detach=False,
-                port=(),
-                org=None,
-                args=net_args,
-            )
+        output = run_python_internal_get_output(
+            ctx,
+            get_impl_path("net.py"),
+            dut,
+            image="",
+            env=(),
+            passenv=(),
+            kill=False,
+            download=(),
+            allow_overwrite=False,
+            signum="SIGTERM",
+            timeout=30,  # 30 second timeout to prevent hanging
+            detach=False,
+            port=(),
+            org=None,
+            args=net_args,
+        )
+        return output.decode('utf-8') if isinstance(output, bytes) else output
     except SystemExit:
-        pass
-    return buf.getvalue()
+        return ""
 
 
 def _resolve_dut(ctx: click.Context, dut_opt: Optional[str] = None) -> str:
@@ -516,21 +515,11 @@ def create_cmd(ctx, name, role, channel, address, box, dut):
     • Net names must be globally unique (regardless of type).
     • No duplicate (role, instrument, channel, address).
     """
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Check if box is a local DUT name first
-    if box:
-        local_ip = get_dut_ip(box)
-        if local_ip:
-            gateway = local_ip
-    elif dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            gateway = local_ip
-        else:
-            gateway = dut
-    else:
-        gateway = get_default_gateway(ctx)
+    # Resolve and validate the box/dut name
+    box_name = box or dut
+    gateway = resolve_and_validate_dut(ctx, box_name)
 
     def _run_and_json(path: str, args: tuple[str, ...] = ()) -> list:
         buf = io.StringIO()
