@@ -133,15 +133,17 @@ def _run_script(ctx: click.Context, script: str, dut: str, *args) -> str:
         pass
     return buf.getvalue()
 
-def _save_nets_batch(ctx: click.Context, dut: str, nets: list["Net"]) -> bool:
+def _save_nets_batch(ctx: click.Context, dut: str, nets: list["Net"], custom_names: dict[str, str] | None = None) -> bool:
     """Save multiple nets using batch save with fallback to individual saves."""
     if not nets:
         return True
 
+    custom_names = custom_names or {}
     nets_data = []
     for n in nets:
+        net_name = custom_names.get(n.key(), n.net)
         nets_data.append({
-            "name": n.net,
+            "name": net_name,
             "role": n.type,
             "address": n.addr,
             "instrument": n.instrument,
@@ -164,8 +166,9 @@ def _save_nets_batch(ctx: click.Context, dut: str, nets: list["Net"]) -> bool:
     saved_count = 0
     for n in nets:
         try:
+            net_name = custom_names.get(n.key(), n.net)
             _run_script(ctx, "net.py", dut, "save", json.dumps({
-                "name": n.net,
+                "name": net_name,
                 "role": n.type,
                 "address": n.addr,
                 "instrument": n.instrument,
@@ -212,14 +215,16 @@ class Net:
             "[Delete]",
         ]
 
-    def as_row_add(self, chosen: bool) -> list[str]:
+    def as_row_add(self, chosen: bool, custom_name: str | None = None) -> list[str]:
+        display_name = custom_name if custom_name else self.net
         return [
             "[SELECTED]" if chosen else "[ADD]",
-            self.net,
+            display_name,
             self.type.upper(),
             self.instrument.replace("_", " "),
             self.chan,
             self.addr,
+            "[Rename]",
         ]
 
     def key(self) -> str:
@@ -365,6 +370,84 @@ class RenameDialog(Screen):
         app._sync_saved_from_disk()
         app._refresh_table()
 
+class RenameNewNetDialog(Screen):
+    """Prompt + text box to enter a new name for an unsaved net in AddScreen."""
+
+    def __init__(self, net: Net, add_screen: "AddScreen") -> None:
+        super().__init__()
+        self.net = net
+        self.add_screen = add_screen
+        self.input: Input
+
+    def compose(self) -> ComposeResult:
+        with Vertical(classes="dialog"):
+            yield Static("Rename Net Before Adding", classes="dialog-title")
+            current_name = self.add_screen.custom_names.get(self.net.key(), self.net.net)
+            yield Static(
+                f"Current name: {current_name}\n"
+                f"Type: {self.net.type}\n"
+                f"Instrument: {self.net.instrument}",
+                classes="dialog-content"
+            )
+            self.input = Input(
+                placeholder="Enter new net name...",
+                id="rename_new_input",
+                value=current_name
+            )
+            yield self.input
+            with Horizontal(classes="dialog-buttons"):
+                yield Button("Cancel", id="cancel")
+                yield Button("Rename", id="confirm", variant="success")
+
+    def on_mount(self) -> None:
+        self.input.focus()
+
+    def on_button_pressed(self, event: Button.Pressed) -> None:
+        """Handle Cancel / Confirm in the rename dialog."""
+        app: NetApp = self.app  # type: ignore[attr-defined]
+
+        if event.button.id == "cancel":
+            app.pop_screen()
+            return
+
+        new_name = self.input.value.strip()
+        current_name = self.add_screen.custom_names.get(self.net.key(), self.net.net)
+
+        if not new_name or new_name == current_name:
+            app.pop_screen()
+            return
+
+        # Check if the name conflicts with any saved net or other custom names
+        if any(n.saved and n.net.lower() == new_name.lower() for n in app.nets):
+            self.input.placeholder = "That name is already used by a saved net!"
+            self.input.value = ""
+            self.input.focus()
+            return
+
+        # Check if the name conflicts with other custom names in the add screen
+        for key, custom in self.add_screen.custom_names.items():
+            if key != self.net.key() and custom.lower() == new_name.lower():
+                self.input.placeholder = "That name is already used in add list!"
+                self.input.value = ""
+                self.input.focus()
+                return
+
+        # Store the custom name
+        self.add_screen.custom_names[self.net.key()] = new_name
+
+        # Update the table display
+        try:
+            tbl = self.add_screen.query_one("#add_tbl", DataTable)
+            # Find the row for this net and update the name column
+            for row_idx, row_key in enumerate(tbl.rows):
+                if row_key.value == self.net.key():
+                    tbl.update_cell_at((row_idx, 1), new_name)
+                    break
+        except NoMatches:
+            pass
+
+        app.pop_screen()
+
 class JLinkDeviceTypeDialog(Screen):
     def __init__(
         self,
@@ -416,6 +499,7 @@ class AddScreen(Screen):
         self.nets: list[Net] = nets
         self.multi_labjack: bool = multi_labjack
         self.chosen: set[str] = set()
+        self.custom_names: dict[str, str] = {}  # key -> custom_name
 
     def _row_allowed(self, n: Net) -> bool:
         """
@@ -423,10 +507,12 @@ class AddScreen(Screen):
         """
         if n.saved:
             return False
-        # Hide exact duplicates of saved nets
-        for s in self.nets:
-            if s.saved and (s.type, s.instrument, s.chan, s.addr) == (n.type, n.instrument, n.chan, n.addr):
-                return False
+
+        # Hide unsaved nets if a saved net already exists for the same physical channel
+        # Key: (type, instrument, channel, address)
+        if any(s.saved and s.type == n.type and s.instrument == n.instrument and s.chan == n.chan and s.addr == n.addr for s in self.nets):
+            return False
+
         # Hide a second net for single-channel instruments (e.g., Keithley 2281S)
         if n.instrument == "Keithley_2281S":
             if any(s.saved and s.instrument == "Keithley_2281S" and s.addr == n.addr for s in self.nets):
@@ -449,21 +535,42 @@ class AddScreen(Screen):
         """
         warnings: list[str] = []
 
-        # Deduplicate nets by unique key (type/instrument/channel/address)
-        uniq: dict[str, Net] = {}
+        # Deduplicate nets by (instrument, channel, address) - NOT by name
+        # This ensures saved "hahahah" on channel 2 deduplicates with unsaved "usb2" on channel 2
+        # Prioritize unsaved nets over saved ones when deduplicating
+        uniq: dict[tuple[str, str, str, str], Net] = {}
         for n in self.nets:
-            uniq.setdefault(n.key(), n)
+            # Key by (type, instrument, channel, address) - excludes name
+            dedup_key = (n.type, n.instrument, n.chan, n.addr)
+            # Only add if key doesn't exist, OR if this net is unsaved and existing is saved
+            if dedup_key not in uniq or (not n.saved and uniq[dedup_key].saved):
+                uniq[dedup_key] = n
         nets: list[Net] = list(uniq.values())
 
         # Detect multiple physical hubs of same type (LabJack, Acroname)
-        chan_seen: dict[str, set[str]] = defaultdict(set)
-        duplicate_hubs: set[str] = set()
+        # Key by (instrument, address) to distinguish different physical devices
+        chan_seen: dict[tuple[str, str], set[str]] = defaultdict(set)
+        duplicate_hubs: set[tuple[str, str]] = set()
         for n in nets:
             if n.instrument in _MULTI_HUBS:
-                if n.chan in chan_seen[n.instrument]:
-                    duplicate_hubs.add(n.instrument)
-                chan_seen[n.instrument].add(n.chan)
-        blocked_families = duplicate_hubs.copy()
+                device_key = (n.instrument, n.addr)
+                if n.chan in chan_seen[device_key]:
+                    # Same channel seen twice on same device - this is expected (saved + unsaved)
+                    # Only block if we see the SAME channel on DIFFERENT devices
+                    pass
+                chan_seen[device_key].add(n.chan)
+
+        # Check if we have multiple devices of the same type (different addresses)
+        device_counts: dict[str, set[str]] = defaultdict(set)
+        for n in nets:
+            if n.instrument in _MULTI_HUBS:
+                device_counts[n.instrument].add(n.addr)
+
+        # Block instrument families that have multiple physical devices
+        blocked_families: set[str] = set()
+        for inst, addrs in device_counts.items():
+            if len(addrs) > 1:
+                blocked_families.add(inst)
 
         remaining: list[Net] = []
         dup_single: set[tuple[str, str]] = set()
@@ -506,6 +613,10 @@ class AddScreen(Screen):
 
         remaining.sort(key=sort_key)
 
+        # Store the nets that are actually displayed in the table for rename lookups
+        # CRITICAL: Only store unsaved nets to prevent delete dialogs
+        self.displayed_nets = {n.key(): n for n in remaining if not n.saved}
+
         with Vertical(classes="dialog"):
             yield Static("Add Available Nets", classes="dialog-title")
 
@@ -514,8 +625,11 @@ class AddScreen(Screen):
                 for w in warnings:
                     yield Static(f"• {w}", classes="warning")
 
-            if remaining:
-                yield Static(f"Found {len(remaining)} available nets. Click 'Select' to add nets:", classes="dialog-content")
+            # CRITICAL: Filter to only unsaved nets - multiple safety checks
+            unsaved_only = [n for n in remaining if not n.saved]
+
+            if unsaved_only:
+                yield Static(f"Found {len(unsaved_only)} available nets. Click 'Select' to add nets:", classes="dialog-content")
                 tbl = DataTable(id="add_tbl", zebra_stripes=True, show_cursor=True)
                 tbl.add_column("[Select]", width=12)
                 tbl.add_column("Name")
@@ -523,8 +637,10 @@ class AddScreen(Screen):
                 tbl.add_column("Instrument")
                 tbl.add_column("Channel")
                 tbl.add_column("Address")
-                for n in remaining:
-                    tbl.add_row("[ADD]", *n.as_row_add(False)[1:], key=n.key())
+                tbl.add_column("[Rename]", width=10)
+                for n in unsaved_only:
+                    custom_name = self.custom_names.get(n.key())
+                    tbl.add_row(*n.as_row_add(False, custom_name), key=n.key())
                 yield tbl
             else:
                 yield Static("No Available Nets to Add", classes="placeholder")
@@ -538,17 +654,29 @@ class AddScreen(Screen):
                     yield Button("Close", id="close", variant="primary")
 
     def on_data_table_cell_selected(self, event: DataTable.CellSelected) -> None:
-        if event.coordinate.column != 0:
-            return
         key = event.cell_key.row_key.value
-        coord = (event.coordinate.row, 0)
-        tbl = self.query_one(DataTable)
-        if key in self.chosen:
-            self.chosen.remove(key)
-            tbl.update_cell_at(coord, "[ADD]")
-        else:
-            self.chosen.add(key)
-            tbl.update_cell_at(coord, "[SELECTED]")
+        col = event.coordinate.column
+
+        # CRITICAL: Stop event propagation to prevent main screen from handling it
+        event.stop()
+
+        # Handle [Select] column (column 0)
+        if col == 0:
+            coord = (event.coordinate.row, 0)
+            tbl = self.query_one(DataTable)
+            if key in self.chosen:
+                self.chosen.remove(key)
+                tbl.update_cell_at(coord, "[ADD]")
+            else:
+                self.chosen.add(key)
+                tbl.update_cell_at(coord, "[SELECTED]")
+
+        # Handle [Rename] column (column 6)
+        elif col == 6:
+            # Get the net from the displayed nets (guaranteed to be unsaved)
+            net = self.displayed_nets.get(key)
+            if net and not net.saved:  # Double-check it's unsaved
+                self.app.push_screen(RenameNewNetDialog(net, self))
 
     def on_button_pressed(self, event: Button.Pressed) -> None:
         """Handle *Cancel* / *Confirm* buttons in the Add-dialog."""
@@ -593,7 +721,7 @@ class AddScreen(Screen):
 
         # If no debug nets selected, save immediately using batch save
         if not debug_nets:
-            if _save_nets_batch(main.ctx, main.dut, selected_nets):
+            if _save_nets_batch(main.ctx, main.dut, selected_nets, self.custom_names):
                 main.show_success(f"Successfully added {len(selected_nets)} nets")
             else:
                 main.show_error("Failed to save some nets")
@@ -620,7 +748,7 @@ class AddScreen(Screen):
                 return
             # All debug prompts done – save all pending nets using batch save
             all_nets_to_save = self._pending_normal_nets + self._pending_debug_nets
-            if _save_nets_batch(main.ctx, main.dut, all_nets_to_save):
+            if _save_nets_batch(main.ctx, main.dut, all_nets_to_save, self.custom_names):
                 main.show_success(f"Successfully added {len(all_nets_to_save)} nets")
             else:
                 main.show_error("Failed to save some nets")
@@ -1072,7 +1200,8 @@ class NetApp(App):
         self.set_timer(3.0, lambda: success_msg.remove())
 
     def _ensure_autogen_unsaved(self) -> None:
-        existing_keys = {(n.type, n.instrument, n.chan, n.addr) for n in self.nets}
+        # Track which channels have unsaved placeholders
+        unsaved_keys = {(n.type, n.instrument, n.chan, n.addr) for n in self.nets if not n.saved}
         role_counter: dict[str, int] = defaultdict(int)
         idx_re = re.compile(r"^([A-Za-z]+)(\d+)$")
 
@@ -1082,7 +1211,7 @@ class NetApp(App):
             if m and _first_word(n.type) == m.group(1):
                 role_counter[n.type] = max(role_counter[n.type], int(m.group(2)))
 
-        # Add placeholder unsaved nets for any missing channels
+        # Add placeholder unsaved nets for ALL channels (even if saved versions exist)
         for dev in self.inst_list:
             instr = dev.get("name", "Unknown")
             addr = dev.get("address", "NA")
@@ -1092,12 +1221,13 @@ class NetApp(App):
                 sorted_channels = sorted(channels, key=lambda ch: str(ch))
                 for ch in sorted_channels:
                     key = (role, instr, ch, addr)
-                    if key in existing_keys:
+                    # Only skip if an UNSAVED placeholder already exists
+                    if key in unsaved_keys:
                         continue
                     role_counter[role] += 1
                     auto_name = f"{_first_word(role)}{role_counter[role]}"
                     self.nets.append(Net(instr, ch, role, auto_name, addr, saved=False))
-                    existing_keys.add(key)
+                    unsaved_keys.add(key)
 
     def on_data_table_cell_selected(self, event: DataTable.CellSelected) -> None:
         key = event.cell_key.row_key.value  # type: ignore[attr-defined]
