@@ -296,14 +296,13 @@ def install(ctx, box, dut, yes, packages):
         lager pip install numpy==1.24.0
         lager pip install scipy matplotlib
     """
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Determine target DUT
-    target = dut or box
-    if not target:
-        target = get_default_gateway(ctx)
-    if not target:
-        click.secho('Error: No Lagerbox specified. Use --box <ip> or set a default.', fg='red', err=True)
-        sys.exit(1)
+    # Determine target DUT (box takes precedence over dut)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    target = resolve_and_validate_dut(ctx, box_name)
 
     # Read current requirements from gateway
     current_packages = _read_remote_requirements(ctx, target)
@@ -364,14 +363,13 @@ def list(ctx, box, dut):
     Shows packages that have been installed via 'lager pip install'.
     These packages persist across container restarts.
     """
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Determine target DUT
-    target = dut or box
-    if not target:
-        target = get_default_gateway(ctx)
-    if not target:
-        click.secho('Error: No Lagerbox specified. Use --box <ip> or set a default.', fg='red', err=True)
-        sys.exit(1)
+    # Determine target DUT (box takes precedence over dut)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    target = resolve_and_validate_dut(ctx, box_name)
 
     # Read packages from gateway
     packages = _read_remote_requirements(ctx, target)
@@ -404,14 +402,13 @@ def uninstall(ctx, box, dut, yes, packages):
         lager pip uninstall pandas
         lager pip uninstall numpy scipy
     """
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Determine target DUT
-    target = dut or box
-    if not target:
-        target = get_default_gateway(ctx)
-    if not target:
-        click.secho('Error: No Lagerbox specified. Use --box <ip> or set a default.', fg='red', err=True)
-        sys.exit(1)
+    # Determine target DUT (box takes precedence over dut)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    target = resolve_and_validate_dut(ctx, box_name)
 
     # Read current requirements from gateway
     current_packages = _read_remote_requirements(ctx, target)
@@ -470,14 +467,13 @@ def apply(ctx, box, dut, yes):
         lager pip apply --box TEST-2
         lager pip apply --box TEST-2 --yes
     """
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Determine target DUT
-    target = dut or box
-    if not target:
-        target = get_default_gateway(ctx)
-    if not target:
-        click.secho('Error: No Lagerbox specified. Use --box <ip> or set a default.', fg='red', err=True)
-        sys.exit(1)
+    # Determine target DUT (box takes precedence over dut)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    target = resolve_and_validate_dut(ctx, box_name)
 
     # Read current packages to show what will be applied
     packages = _read_remote_requirements(ctx, target)
