@@ -174,7 +174,12 @@ def run_python_internal(ctx, runnable, box, image, env, passenv, kill, download,
         post_data.append(('timeout', timeout))
 
     if os.path.isfile(runnable):
-        post_data.append(('script', open(runnable, 'rb')))
+        with open(runnable, 'rb') as f:
+            script_content = f.read()
+        # Use BytesIO to create a file-like object that can be read multiple times
+        import io
+        script_file = io.BytesIO(script_content)
+        post_data.append(('script', (os.path.basename(runnable), script_file, 'application/octet-stream')))
     elif os.path.isdir(runnable):
         try:
             max_content_size = MAX_ZIP_SIZE * 2
@@ -192,6 +197,21 @@ def run_python_internal(ctx, runnable, box, image, env, passenv, kill, download,
         raise ValueError(f'Could not find runnable {runnable}')
 
     resp = session.run_python(gateway, files=post_data)
+
+    # Check for HTTP errors before trying to parse streaming response
+    if resp.status_code >= 400:
+        click.secho(f'Gateway returned error (HTTP {resp.status_code})', fg='red', err=True)
+        try:
+            # Try to extract error message from JSON response
+            error_data = resp.json()
+            if 'error' in error_data:
+                click.secho(f'Error: {error_data["error"]}', fg='red', err=True)
+            else:
+                click.echo(resp.text, err=True)
+        except Exception:
+            click.echo(resp.text, err=True)
+        ctx.exit(1)
+
     kill_python = functools.partial(session.kill_python, gateway, lager_process_id)
     handler = functools.partial(sigint_handler, kill_python)
     signal.signal(signal.SIGINT, handler)
@@ -242,20 +262,11 @@ def run_python_internal(ctx, runnable, box, image, env, passenv, kill, download,
 @click.argument('args', nargs=-1)
 def python(ctx, runnable, box, dut, image, env, passenv, kill, download, allow_overwrite, signum, timeout, detach, port, org, add_file, args):
     """Run Python script on gateway"""
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Check if box is a local DUT name first
-    gateway = box or dut
-    if box:
-        local_ip = get_dut_ip(box)
-        if local_ip:
-            gateway = local_ip
-    elif dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            gateway = local_ip
-        else:
-            gateway = dut
+    # Resolve and validate the box/dut name
+    box_name = box or dut
+    gateway = resolve_and_validate_dut(ctx, box_name)
 
     if not runnable and not kill:
         raise click.UsageError('Please supply a RUNNABLE or the --kill option')
