@@ -29,21 +29,11 @@ def _require_netname(ctx) -> str:
 
 
 def _resolve_gateway(ctx, box, dut):
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Check if box is a local DUT name first
-    if box:
-        local_ip = get_dut_ip(box)
-        if local_ip:
-            return local_ip
-
-    # Check if dut is a local DUT name
-    if dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            return local_ip
-
-    return box or dut or get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+    return resolve_and_validate_dut(ctx, box_name)
 
 
 def _run_net_py(ctx: click.Context, dut: str, *args: str) -> list[dict]:
@@ -272,7 +262,7 @@ def period(ctx, mcu, box, dut, display, cursor):
     run_python_internal(
         ctx,
         get_impl_path("measurement.py"),
-        dut,
+        gateway,
         image="",
         env=(f"LAGER_COMMAND_DATA={json.dumps(data)}",),
         passenv=(),
