@@ -4,7 +4,7 @@ import click
 import json
 from ..context import get_default_gateway, get_impl_path, get_default_net
 from ..python.commands import run_python_internal
-from ..dut_storage import get_dut_ip
+from ..dut_storage import resolve_and_validate_dut
 
 ###############################################################################
 # Internal helpers                                                            #
@@ -18,14 +18,10 @@ def _run_backend(ctx: click.Context, gateway: str | None, action: str, **params)
         raise click.UsageError("NETNAME required for solar command.")
     # Determine target gateway (if any)
     if gateway is None:
-        gateway = ctx.obj.gateway or get_default_gateway(ctx)
+        gateway = ctx.obj.gateway
 
-    # Resolve DUT name to IP address if it's not already an IP address
-    if gateway and not gateway.replace('.', '').isdigit():
-        # Try to resolve gateway name to IP from .lager file
-        resolved_ip = get_dut_ip(gateway)
-        if resolved_ip:
-            gateway = resolved_ip
+    # Resolve and validate the gateway name
+    gateway = resolve_and_validate_dut(ctx, gateway)
 
     # Prepare command data for solar backend
     data = {
