@@ -11,6 +11,7 @@ import json
 from contextlib import redirect_stdout
 
 import click
+import requests
 from texttable import Texttable
 
 from ..context import get_default_gateway, get_impl_path, get_default_net
@@ -26,59 +27,41 @@ def _resolve_gateway(ctx, box, dut):
     Resolve gateway/dut parameter to IP address.
     Returns tuple of (ip_address, dut_name) where dut_name is used for username lookup.
     """
-    from ..dut_storage import get_dut_ip, get_dut_name_by_ip
+    from ..dut_storage import resolve_and_validate_dut, get_dut_name_by_ip
 
-    # Check if box is a local DUT name first
-    if box:
-        local_ip = get_dut_ip(box)
-        if local_ip:
-            return (local_ip, gateway)  # Return IP and DUT name for username lookup
-        # If gateway is an IP, try to find its DUT name
-        dut_name = get_dut_name_by_ip(gateway)
-        return (gateway, dut_name)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
 
-    # Check if dut is a local DUT name
-    if dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            return (local_ip, dut)  # Return IP and DUT name for username lookup
-        # If dut is an IP, try to find its DUT name
-        dut_name = get_dut_name_by_ip(dut)
-        return (dut, dut_name)
+    # Resolve and validate the box/dut name
+    resolved_ip = resolve_and_validate_dut(ctx, box_name)
 
-    # Fallback to default gateway
-    default_gw = get_default_gateway(ctx)
-    dut_name = get_dut_name_by_ip(default_gw) if default_gw else None
-    return (default_gw, dut_name)
+    # Try to find DUT name for username lookup
+    # If box_name was provided and is not an IP, it's the DUT name
+    if box_name and not box_name.replace('.', '').isdigit():
+        dut_name = box_name
+    else:
+        # It was an IP, try reverse lookup
+        dut_name = get_dut_name_by_ip(resolved_ip)
+
+    return (resolved_ip, dut_name)
 
 
 def _run_net_py(ctx: click.Context, dut: str, *args: str) -> list[dict]:
-    buf = io.StringIO()
+    """
+    Fetch nets list from the gateway.
+    Uses the HTTP endpoint on port 9000 (Python container).
+    """
     try:
-        with redirect_stdout(buf):
-            run_python_internal(
-                ctx,
-                get_impl_path("net.py"),
-                dut,
-                image="",
-                env={},
-                passenv=(),
-                kill=False,
-                download=(),
-                allow_overwrite=False,
-                signum="SIGTERM",
-                timeout=0,
-                detach=False,
-                port=(),
-                org=None,
-                args=args or ("list",),
-            )
-    except SystemExit:
-        pass
-    raw = buf.getvalue() or "[]"
-    try:
-        return json.loads(raw)
-    except json.JSONDecodeError:
+        gateway_url = f'http://{dut}:9000/uart/nets/list'
+        response = requests.get(gateway_url, timeout=5)
+        if response.status_code == 200:
+            data = response.json()
+            return data.get('nets', [])
+        else:
+            click.echo(f"Error: Gateway returned status {response.status_code}", err=True)
+            return []
+    except (requests.RequestException, json.JSONDecodeError) as e:
+        click.echo(f"Error: Could not fetch nets from gateway at {dut}:9000: {e}", err=True)
         return []
 
 
@@ -89,7 +72,7 @@ def _list_uart_nets(ctx, box):
 
 def _get_uart_net(ctx, box, netname):
     """Get a specific UART net by name"""
-    nets = _list_uart_nets(ctx, gateway)
+    nets = _list_uart_nets(ctx, box)
     for net in nets:
         if net.get("name") == netname:
             return net
@@ -144,7 +127,7 @@ def _find_device_path(usb_serial: str, inst_list: list[dict]) -> str | None:
 
 def display_nets(ctx, box, netname: str | None):
     """Display UART nets with their configuration parameters."""
-    uart_nets = _list_uart_nets(ctx, gateway)
+    uart_nets = _list_uart_nets(ctx, box)
 
     # Check if there are any UART nets to display
     if not uart_nets:
@@ -152,7 +135,7 @@ def display_nets(ctx, box, netname: str | None):
         return
 
     # Query instruments to get current device paths
-    inst_list = _run_query_instruments(ctx, gateway)
+    inst_list = _run_query_instruments(ctx, box)
 
     table = Texttable()
     table.set_deco(Texttable.HEADER)
@@ -231,93 +214,43 @@ def _run_backend(ctx, dut, action: str, **params):
 
 def _connect_uart_http(ctx, gateway, netname, overrides, interactive):
     """
-    Connect to UART via HTTP using run_python_internal().
-    Uses the UART dispatcher on the gateway for net-based configuration.
+    Connect to UART via WebSocket (both read-only and interactive modes).
+
+    Both modes now use WebSocket for reliability - no more HTTP streaming crashes!
 
     Args:
         ctx: Click context
         gateway: Gateway IP address
         netname: Name of the UART net to connect to
         overrides: Dictionary of serial port parameter overrides
-        interactive: Whether to use interactive mode
+        interactive: Whether to use interactive mode (bidirectional with keyboard input)
     """
-    import tempfile
-    import os
+    import time
 
-    # Build Python script that uses the UART dispatcher
-    action = "monitor_interactive" if interactive else "monitor"
+    # Both interactive and read-only modes now use WebSocket
+    if interactive:
+        from .websocket_client import connect_uart_interactive
+        gateway_url = f'http://{gateway}:9000'
+        connect_func = connect_uart_interactive
+    else:
+        from .websocket_client import connect_uart_readonly
+        gateway_url = f'http://{gateway}:9000'
+        connect_func = connect_uart_readonly
 
-    # Convert overrides dict to Python dict literal
-    overrides_str = repr(overrides)
-
-    # Create the UART script content that uses the gateway's UART dispatcher
-    uart_script = f'''
-import sys
-sys.path.insert(0, '/app/gateway_python')
-
-from lager.uart.dispatcher import {action}
-
-try:
-    {action}({repr(netname)}, overrides={overrides_str})
-except KeyboardInterrupt:
-    print("\\n\\033[31mDisconnected\\033[0m", file=sys.stderr)
-    sys.exit(0)
-except Exception as e:
-    print(f"\\033[31mError: {{e}}\\033[0m", file=sys.stderr)
-    sys.exit(1)
-'''
-
-    # Write script to temporary file
-    with tempfile.NamedTemporaryFile(mode='w', suffix='.py', delete=False) as f:
-        f.write(uart_script)
-        temp_script = f.name
-
-    try:
-        # Set terminal to raw mode for interactive sessions
-        old_tty_settings = None
-        if interactive:
-            try:
-                import termios
-                import tty
-                old_tty_settings = termios.tcgetattr(sys.stdin.fileno())
-                tty.setcbreak(sys.stdin.fileno())
-            except Exception:
-                old_tty_settings = None
-
-        # Use run_python_internal to execute the script on the gateway
-        # This streams output over HTTP (same pattern as all other lager commands)
-        run_python_internal(
-            ctx,
-            temp_script,
-            gateway,
-            image='',
-            env=(),
-            passenv=(),
-            kill=False,
-            download=(),
-            allow_overwrite=False,
-            signum='SIGTERM',
-            timeout=0,
-            detach=False,
-            port=(),
-            org=None,
-            args=(),
-        )
-    except KeyboardInterrupt:
-        click.secho('\nDisconnected', fg='red', err=True)
-    finally:
-        # Restore terminal settings
-        if old_tty_settings:
-            try:
-                import termios
-                termios.tcsetattr(sys.stdin.fileno(), termios.TCSADRAIN, old_tty_settings)
-            except Exception:
-                pass
-        # Clean up temporary file
+    # Retry logic for WebSocket connection
+    max_retries = 2
+    for attempt in range(max_retries):
         try:
-            os.unlink(temp_script)
-        except Exception:
-            pass
+            exit_code = connect_func(gateway_url, netname, overrides)
+            ctx.exit(exit_code)
+            return
+        except Exception as e:
+            if attempt < max_retries - 1:
+                if "Connection refused" in str(e) or "Failed to establish" in str(e):
+                    click.echo(f"Connection failed, retrying in 2 seconds... (attempt {attempt + 1}/{max_retries})", err=True)
+                    time.sleep(2)
+                    continue
+            raise
 
 
 # ---------- CLI ----------
