@@ -16,9 +16,10 @@ from ..context import get_default_gateway
 @click.option('--dut', required=False, hidden=True, help='DUT name or IP to update')
 @click.option('--yes', is_flag=True, help='Skip confirmation prompt')
 @click.option('--skip-restart', is_flag=True, help='Skip container restart after update')
-@click.option('--branch', default='main', help='Git branch to pull from (default: main)')
+@click.option('--version', required=False, help='Gateway version/branch to update to (e.g., staging, main)')
+@click.option('--branch', required=False, hidden=True, help='(Deprecated: use --version) Git branch to pull from')
 @click.option('--check-jlink', is_flag=True, help='Check for J-Link and offer to install if missing')
-def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
+def update(ctx, box, dut, yes, skip_restart, version, branch, check_jlink):
     """
     Update gateway code on a box from GitHub repository
 
@@ -30,13 +31,28 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     5. Restart Docker containers to apply changes
     6. Verify services are running correctly
     7. (Optional) Check for J-Link and offer to install if missing
+    8. Store version information on gateway and in local config
 
     Example:
         lager update --box JUL-3
         lager update --box HYP-3 --yes
         lager update --box JUL-FRESH --check-jlink
-        lager update --box JUL-3 --branch staging
+        lager update --box JUL-3 --version staging
     """
+    from ..dut_storage import update_dut_version
+    from .. import __version__ as cli_version
+
+    # Handle version/branch compatibility (version takes precedence)
+    if not version and not branch:
+        # Default to main if neither specified
+        target_version = 'main'
+    elif version:
+        # Use --version if provided
+        target_version = version
+    else:
+        # Fall back to --branch for backward compatibility
+        target_version = branch
+
     # Use box or dut (box takes precedence)
     resolved = box or dut
 
@@ -58,7 +74,8 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     click.echo()
     click.secho('Gateway Update', fg='blue', bold=True)
     click.echo(f'Target:  {dut} ({resolved_dut})')
-    click.echo(f'Branch:  {branch}')
+    click.echo(f'Version: {target_version}')
+    click.echo(f'CLI:     {cli_version}')
     click.echo()
 
     # Confirm before proceeding
@@ -120,9 +137,9 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
         click.echo(' (unknown)')
 
     # Step 4: Fetch and check for updates
-    click.echo(f'Fetching updates from origin/{branch}...', nl=False)
+    click.echo(f'Fetching updates from origin/{target_version}...', nl=False)
     result = subprocess.run(
-        ['ssh', ssh_host, f'cd ~/gateway && git fetch origin {branch}'],
+        ['ssh', ssh_host, f'cd ~/gateway && git fetch origin {target_version}'],
         capture_output=True,
         text=True
     )
@@ -141,7 +158,7 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
 
     # Check if there are updates available
     result = subprocess.run(
-        ['ssh', ssh_host, f'cd ~/gateway && git rev-list HEAD..origin/{branch} --count'],
+        ['ssh', ssh_host, f'cd ~/gateway && git rev-list HEAD..origin/{target_version} --count'],
         capture_output=True,
         text=True
     )
@@ -157,35 +174,38 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
             needs_pull = True
 
     if needs_pull:
-        # Step 5: Ensure udev_rules is tracked in sparse checkout
-        click.echo('Ensuring udev_rules is tracked...', nl=False)
+        # Step 5: Ensure udev_rules and cli/__init__.py are tracked in sparse checkout
+        click.echo('Ensuring required files are tracked...', nl=False)
         result = subprocess.run(
-            ['ssh', ssh_host, 'cd ~/gateway && git sparse-checkout list | grep -q "^udev_rules$" || git sparse-checkout add udev_rules'],
+            ['ssh', ssh_host,
+             'cd ~/gateway && '
+             'git sparse-checkout list | grep -q "^udev_rules$" || git sparse-checkout add udev_rules && '
+             'git sparse-checkout list | grep -q "^cli/__init__.py$" || git sparse-checkout add cli/__init__.py'],
             capture_output=True,
             text=True
         )
         # Ignore errors - sparse checkout add is idempotent and may not be needed
         click.secho(' OK', fg='green')
 
-        # Step 6: Checkout the specified branch
-        click.echo(f'Checking out branch {branch}...', nl=False)
+        # Step 6: Checkout the specified version/branch
+        click.echo(f'Checking out version {target_version}...', nl=False)
         result = subprocess.run(
-            ['ssh', ssh_host, f'cd ~/gateway && git checkout {branch}'],
+            ['ssh', ssh_host, f'cd ~/gateway && git checkout {target_version}'],
             capture_output=True,
             text=True
         )
         if result.returncode != 0:
             click.secho(' FAILED', fg='red')
-            click.secho(f'Error: Failed to checkout branch {branch}', fg='red', err=True)
+            click.secho(f'Error: Failed to checkout version {target_version}', fg='red', err=True)
             if result.stderr:
                 click.echo(result.stderr, err=True)
             ctx.exit(1)
         click.secho(' OK', fg='green')
 
         # Step 7: Reset to match remote (force pull, handles divergent branches)
-        click.echo(f'Updating to match origin/{branch}...', nl=False)
+        click.echo(f'Updating to match origin/{target_version}...', nl=False)
         result = subprocess.run(
-            ['ssh', ssh_host, f'cd ~/gateway && git reset --hard origin/{branch}'],
+            ['ssh', ssh_host, f'cd ~/gateway && git reset --hard origin/{target_version}'],
             capture_output=True,
             text=True
         )
@@ -275,10 +295,9 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
         copy_result = subprocess.run(
             ['ssh', ssh_host,
              'cd ~/gateway && '
-             'cp -rf gateway/python . 2>/dev/null && '
-             'cp -rf gateway/controller . 2>/dev/null && '
+             'cp -rf gateway/lager . 2>/dev/null && '
              'cp -rf gateway/udev_rules . 2>/dev/null && '
-             'cp -f gateway/start_all_containers.sh . 2>/dev/null && '
+             'cp -f gateway/start_lager.sh . 2>/dev/null && '
              'cp -f gateway/start_all_containers_hyphen.sh . 2>/dev/null || true'],
             capture_output=True,
             text=True,
@@ -316,7 +335,7 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
         click.echo()
         click.secho('Skipping container restart (--skip-restart flag set)', fg='yellow')
         click.echo('Run this manually to apply changes:')
-        click.echo(f'  ssh {ssh_host} "cd ~/gateway && ./start_all_containers.sh"')
+        click.echo(f'  ssh {ssh_host} "cd ~/gateway && ./start_lager.sh"')
         ctx.exit(0)
 
     click.echo()
@@ -330,7 +349,7 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
          'cd ~/gateway && '
          'docker stop $(docker ps -aq) 2>/dev/null || true && '
          'docker rm $(docker ps -aq) 2>/dev/null || true && '
-         'docker rmi python controller lagerdata/controller 2>/dev/null || true'],
+         'docker rmi lager python controller lagerdata/controller 2>/dev/null || true'],
         capture_output=True,
         text=True,
         timeout=30
@@ -340,15 +359,15 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     else:
         click.secho(' WARNING', fg='yellow')
 
-    # Rebuild Python container with --no-cache to ensure fresh dependencies
-    click.echo('  Rebuilding Python container (no cache)...')
+    # Rebuild Lager container with --no-cache to ensure fresh dependencies
+    click.echo('  Rebuilding Lager container (no cache)...')
     click.echo()
 
     # Stream build output to show progress
     process = subprocess.Popen(
         ['ssh', ssh_host,
-         'cd ~/gateway/python && '
-         'docker build --no-cache -f docker/gatewaypy3.Dockerfile -t python .'],
+         'cd ~/gateway/lager && '
+         'docker build --no-cache -f docker/gatewaypy3.Dockerfile -t lager .'],
         stdout=subprocess.PIPE,
         stderr=subprocess.STDOUT,
         text=True,
@@ -364,45 +383,15 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
 
     click.echo()
     if return_code != 0:
-        click.secho('  Python container rebuild FAILED', fg='red', bold=True)
-        click.secho('Error: Failed to rebuild Python container', fg='red', err=True)
+        click.secho('  Lager container rebuild FAILED', fg='red', bold=True)
+        click.secho('Error: Failed to rebuild Lager container', fg='red', err=True)
         ctx.exit(1)
-    click.secho('  Python container rebuild complete!', fg='green', bold=True)
-    click.echo()
-
-    # Rebuild Controller container (can use cache)
-    click.echo('  Rebuilding Controller container...')
-    click.echo()
-
-    # Stream build output to show progress
-    process = subprocess.Popen(
-        ['ssh', ssh_host,
-         'cd ~/gateway/controller && '
-         './build_controller_container.sh'],
-        stdout=subprocess.PIPE,
-        stderr=subprocess.STDOUT,
-        text=True,
-        bufsize=1
-    )
-
-    # Stream output line by line
-    if process.stdout:
-        for line in process.stdout:
-            click.echo(f'    {line}', nl=False)
-
-    return_code = process.wait(timeout=600)
-
-    click.echo()
-    if return_code != 0:
-        click.secho('  Controller container rebuild FAILED', fg='red', bold=True)
-        click.secho('Error: Failed to rebuild Controller container', fg='red', err=True)
-        ctx.exit(1)
-    click.secho('  Controller container rebuild complete!', fg='green', bold=True)
+    click.secho('  Lager container rebuild complete!', fg='green', bold=True)
 
     # Start containers using the startup script
     click.echo('  Starting containers...', nl=False)
     result = subprocess.run(
-        ['ssh', ssh_host, 'cd ~/gateway && ./start_all_containers.sh'],
+        ['ssh', ssh_host, 'cd ~/gateway && chmod +x start_lager.sh && ./start_lager.sh'],
         capture_output=True,
         text=True,
         timeout=60
@@ -416,7 +405,7 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
         click.echo()
         click.echo('You may need to manually restart containers:')
         click.echo(f'  ssh {ssh_host}')
-        click.echo('  cd ~/gateway && ./start_all_containers.sh')
+        click.echo('  cd ~/gateway && chmod +x start_lager.sh && ./start_lager.sh')
         ctx.exit(1)
 
     click.secho(' OK', fg='green')
@@ -430,17 +419,17 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     click.echo('Verifying container status...', nl=False)
     result = subprocess.run(
         ['ssh', ssh_host,
-         "docker ps --filter 'name=controller' --filter 'name=python' --format '{{.Names}}' | wc -l"],
+         "docker ps --filter 'name=lager' --format '{{.Names}}' | wc -l"],
         capture_output=True,
         text=True
     )
 
     if result.returncode == 0:
         running_count = int(result.stdout.strip())
-        if running_count >= 2:
+        if running_count >= 1:
             click.secho(' OK', fg='green')
         else:
-            click.secho(f' WARNING (only {running_count}/2 containers running)', fg='yellow')
+            click.secho(f' WARNING (lager container not running)', fg='yellow')
     else:
         click.secho(' FAILED', fg='red')
 
@@ -449,7 +438,7 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     click.secho('Container Status:', fg='blue', bold=True)
     result = subprocess.run(
         ['ssh', ssh_host,
-         "docker ps --filter 'name=controller' --filter 'name=python' "
+         "docker ps --filter 'name=lager' "
          "--format 'table {{.Names}}\t{{.Status}}'"],
         capture_output=True,
         text=True
@@ -457,6 +446,76 @@ def update(ctx, box, dut, yes, skip_restart, branch, check_jlink):
     if result.returncode == 0:
         click.echo(result.stdout.strip())
 
+    # Step 12: Read gateway CLI version from the checked-out code
+    click.echo()
+    click.echo('Reading gateway CLI version...', nl=False)
+
+    # Read __version__ from cli/__init__.py on the gateway
+    read_version_cmd = (
+        'cd ~/gateway && '
+        'grep -E "^__version__\\s*=\\s*" cli/__init__.py | '
+        'sed -E "s/__version__\\s*=\\s*[\\x27\\x22]([^\\x27\\x22]+)[\\x27\\x22]/\\1/"'
+    )
+
+    result = subprocess.run(
+        ['ssh', ssh_host, read_version_cmd],
+        capture_output=True,
+        text=True
+    )
+
+    if result.returncode == 0 and result.stdout.strip():
+        gateway_cli_version = result.stdout.strip()
+        click.secho(f' {gateway_cli_version}', fg='green')
+    else:
+        # Fallback to branch name if we can't read the version
+        gateway_cli_version = target_version
+        click.secho(f' (using branch name: {target_version})', fg='yellow')
+
+    # Step 13: Store version information on gateway
+    click.echo('Storing version information...', nl=False)
+
+    # Store the gateway CLI version (not the branch name)
+    # Format: <gateway_version>|<current_local_cli_version>
+    version_content = f'{gateway_cli_version}|{cli_version}'
+
+    # Try to store in /etc/lager/version (should be writable without sudo)
+    # /etc/lager is mounted into the Docker container, so the container can read it
+    store_version_cmd = f'echo "{version_content}" > /etc/lager/version'
+
+    result = subprocess.run(
+        ['ssh', ssh_host, store_version_cmd],
+        capture_output=True,
+        text=True
+    )
+
+    # If direct write fails, try with sudo (fallback for restricted permissions)
+    if result.returncode != 0:
+        sudo_cmd = (
+            f'sudo mkdir -p /etc/lager && '
+            f'echo "{version_content}" | sudo tee /etc/lager/version > /dev/null && '
+            f'sudo chmod 644 /etc/lager/version'
+        )
+        result = subprocess.run(
+            ['ssh', ssh_host, sudo_cmd],
+            capture_output=True,
+            text=True
+        )
+
+    if result.returncode == 0:
+        click.secho(' OK', fg='green')
+
+        # Update version in local DUT storage with gateway CLI version
+        if dut:  # Only if we have a DUT name (not just IP)
+            if update_dut_version(dut, gateway_cli_version):
+                click.echo(f'  Updated local config: {dut} → {gateway_cli_version}')
+            else:
+                # DUT not in local storage (might be using IP directly)
+                pass
+    else:
+        click.secho(' FAILED', fg='red')
+        if result.stderr:
+            click.echo(f'  Error: {result.stderr.strip()[:100]}')
+
     # Check for J-Link installation if requested
     if check_jlink:
         click.echo()
