@@ -13,16 +13,9 @@ def _resolve_gateway(ctx: click.Context, dut: str | None) -> str:
     """
     Resolve DUT name to IP address if it's a local DUT, otherwise return as-is.
     """
-    from ..dut_storage import get_dut_ip
+    from ..dut_storage import resolve_and_validate_dut
 
-    if dut:
-        # Check if this is a local DUT name
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            return local_ip
-        return dut
-
-    return get_default_gateway(ctx)
+    return resolve_and_validate_dut(ctx, dut)
 
 
 def _invoke_remote(
