@@ -22,16 +22,9 @@ def _get_dut_ip_address(ctx: click.Context, dut: str = None) -> str:
     Returns:
         IP address string
     """
-    # Check if dut is a local DUT name
-    if dut:
-        local_ip = get_dut_ip(dut)
-        if local_ip:
-            return local_ip
-        # Otherwise return as-is (could be gateway ID or IP)
-        return dut
+    from ..dut_storage import resolve_and_validate_dut
 
-    # Get default gateway (will prompt if multiple gateways and no default set)
-    return get_default_gateway(ctx)
+    return resolve_and_validate_dut(ctx, dut)
 
 
 def _run_webcam_command(ctx: click.Context, dut_ip: str, action: str, net_name: str = None) -> dict:
