@@ -9,7 +9,7 @@ import json
 from texttable import Texttable
 from ..context import get_default_gateway, get_impl_path
 from ..python.commands import run_python_internal
-from ..dut_storage import get_dut_ip
+from ..dut_storage import resolve_and_validate_dut
 
 @click.group(name='wifi', hidden=True)
 def _wifi():
@@ -27,15 +27,10 @@ def status(ctx, box, dut):
         Get the current WiFi Status of the gateway
     """
     # Use box or dut (box takes precedence)
-    dut = box or dut
+    box_name = box or dut
 
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     status_args = {
         'action': 'status'
@@ -59,13 +54,11 @@ def access_points(ctx, box,
     """
         Get WiFi access points visible to the gateway
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     scan_args = {
         'action': 'scan',
@@ -92,13 +85,11 @@ def connect(ctx, box,
     """
         Connect the gateway to a new network
     """
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     wifi_config = {
         'ssid': ssid,
@@ -129,13 +120,11 @@ def delete_connection(ctx, box,
         click.echo("Aborting")
         return
 
-    # Resolve DUT name to IP if needed
-    if dut:
-        saved_ip = get_dut_ip(dut)
-        if saved_ip:
-            dut = saved_ip
-    else:
-        dut = get_default_gateway(ctx)
+    # Use box or dut (box takes precedence)
+    box_name = box or dut
+
+    # Resolve and validate the box/dut name
+    dut = resolve_and_validate_dut(ctx, box_name)
 
     run_python_internal(
         ctx, get_impl_path('wifi_disconnect.py'), dut,
